﻿/*
        * circularGauge.jsx
        *
         Description: This Adobe Illustrator script draws a series of
         radial strokes, with a number at the end of each. Settings are
         entered through a simple dialog.
        
         Handy for concocting gauges. ::
        * dvb2007
      1 */
      
      #include "../shared/ominoDialogMaker.jsx"
      
      
      // this variable goes outside the main brackets to keep the
      // most recent dialog values.
      var gLastGaugeSettings;
      
      {
      var docCount = documents.length;
      
      if (docCount == 0)
      {
         documents.add();
      }
      
      
      /*
       * Add a point to PathPoints,
      30: * straigh.
       */
      function addP(points,x,y)
      {
          var p = points.add();
          p.anchor = [x,y];
          p.leftDirection = p.anchor;
          p.rightDirection = p.anchor;
      }
      function addP2(group,x1,y1,x2,y2)
      {
          var pathItem = group.pathItems.add();
          var points = pathItem.pathPoints; // brand new, empty
          addP(points,x1,y1);
          addP(points,x2,y2);
      }
      
      function addRadialTick(group,cx,cy,angle,r1,r2)
      {
          var angleR = angle / 57.29577951;
          var x1 = cx + Math.cos(angleR) * r1;
          var y1 = cy + Math.sin(angleR) * r1;
          var x2 = cx + Math.cos(angleR) * r2;
          var y2 = cy + Math.sin(angleR) * r2;
          addP2(group,x1,y1,x2,y2);
      }
      
      function addRadialText(group,cx,cy,angle,r,text,textSize)
      {
          // add the text... a little further out
          var angleR = angle / 57.29577951;
          var x2 = cx + Math.cos(angleR) * (r);
          var y2 = cy + Math.sin(angleR) * (r);
          var textAngle = angle - 90;
          //var t = Math.floor(x2) + "," + Math.floor(y2);
          //t = "" + angle;
          addText(group,text,
                  x2,y2,textSize,textAngle);
      }
      
      function layerByName(layers,name)
      {
          var i;
          for(i = 0; i < layers.length; i++)
          {
              var layer = layers[i];
              if(layer.name == name)
                  return layer;
          }
          return null;
      }
      
      function newLayer(doc,baseName)
      {
          var n = 0;
          var result = null;
          while(result == null)
          {
              n++;
              var aName = "" + baseName + n;
              var existingLayer = layerByName(doc.layers,aName);
              if(existingLayer == null)
              {
                  result = doc.layers.add();
                  result.name = aName;
              }
          }
          return result;
      }
      
     function addRadialTicks(layer,groupName,cx,cy,angleStart,angleEnd,divisionCount,r1,r2)
     {
         var group = layer.groupItems.add();
         group.name = groupName;
     
         if(angleEnd % 360 == angleStart)
         {
             angleEnd -= 360 / divisionCount;
             divisionCount--;
         }
     
         for(var i = 0; i <= divisionCount; i++)
         {
             var theta = angleStart + (angleEnd - angleStart) * i / divisionCount;
             addRadialTick(group,cx,cy,theta,r1,r2);
         }
     }
     
     function addRadialTexts(layer,groupName,cx,cy,angleStart,angleEnd,divisionCount,valueLow,valueHigh,r)
     {
         var group = layer.groupItems.add();
         group.name = groupName;
         var doc = app.documents[0];
         var newLayer = doc.layers.add();
         newLayer.name = name;
     
         if(angleEnd % 360 == angleStart)
         {
             angleEnd -= 360 / divisionCount;
             valueHigh = valueLow + (valueHigh - valueLow) * (divisionCount - 1) / divisionCount;
             divisionCount--;
         }
     
         var textSize;
         {
             var angleStep = (angleEnd - angleStart) / divisionCount;
             var approximateStepLength = angleStep * r / 57.29577951;
             textSize = approximateStepLength / 2; // consider n characters...
         }
     
         for(var i = 0; i <= divisionCount; i++)
         {
             var theta = angleStart + (angleEnd - angleStart) * i / divisionCount;
             
             var v = valueLow + (valueHigh - valueLow) * i / divisionCount;
             addRadialText(group,cx,cy,theta,r,v,textSize);
         }
     }
     
     function addText(group,text,x,y,size,angle)
     {
         var textItem = group.textFrames.add();
         textItem.contents = text;
     
     
         textItem.textRange.characterAttributes.size = size;
     
         textItem.rotate(angle, undefined, undefined, undefined, undefined, Transformation.CENTER);
     
         var bounds = textItem.visibleBounds;
         
         var tW = Math.abs(bounds[2] - bounds[0]);
         var tH = Math.abs(bounds[3] - bounds[1]);
     
         // coords go from bottom to top...
         // but text position is left, top.
         var tX = x - tW/2;
         var tY = y + tH/2;
     
         textItem.position = [tX,tY];
     }
     
     function main()
     {
         var docRef = documents[0];
         var docHeight = docRef.height;
         var docWidth = docRef.width;
     
     
         // default values
         var cx = docWidth / 2;
         var cy = docHeight / 2;
     
         var r1 = docWidth / 4;
         var r2 = docWidth / 3;
         var r3 = docWidth / 2.8;
     
         var a1 = 0;
         var a2 = 180;
     
         var steps = 20;
     
         var omd = newOminoDialog("Circular Gauge");
     
         
     
     
         omd.boxedText(5,"This Illustrator script draws circular gauges."
             + " Each gauge has tick marks and numbered texts. They're put in"
             + " separate groups so it's easy to change their style afterwards.");
         omd.separator();
         omd.number("Center X","cx",cx);
         omd.number("Y","cy",cy);
         omd.gap();
         omd.number("Divisions","steps",steps);
         omd.gap();
         omd.number("Angle Start","a1",a1);
         omd.number("End","a2",a2);
         omd.number("Value Low","v1",0);
         omd.number("High","v2",100);
         omd.number("Inner Radius","r1",r1);
         omd.number("Outer","r2",r2);
         omd.number("Text","r3",r3);
         
         omd.set(gLastGaugeSettings);
         var result = omd.run();
         
         if(result)
             gLastGaugeSettings = result;
     
         if(!result)
             return; // canceled
     
         a1 = result.a1;
         a2 = result.a2;
         cx = result.cx;
         cy = result.cy;
     
         var r1 = 360 * Math.random(100);
         var r2 = 360 * Math.random(100);
         var steps = result.steps;
     
         r1 = result.r1;
         r2 = result.r2;
         r3 = result.r3;
         
         var v1 = result.v1;
         var v2 = result.v2;
     
         var layer = newLayer(app.documents[0],"gauge_");
         addRadialTicks(layer,"ticks",cx,cy,a1,a2,steps,r1,r2);
         addRadialTexts(layer,"texts",cx,cy,a1,a2,steps,v1,v2,r3);
         //addRadialTicks("angleLines",cx,cy,0,360,24,70,110);
         //addRadialTexts("angleTexts",cx,cy,0,360,24,120);
     }
     
     main();
     }
     
     
